/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "util.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#if defined HAVE_STRING_H || defined _LIBC
# include <string.h>
#else
# include <strings.h>
#endif

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
#include "getopt.h"
#endif

#include "file/file.h"
#include "tasks/tasks.h"
#include "ui/uiHandler.h"
#include "snd/soundHandler.h"

static void table_init();
static void table_set (const char*, const char*);
static const char* table_get (const char*);
static void table_close();
static void set_defaults();
static void parse_options (int*, char***);
static void init_files();

static void parse_filename (const char*);
static void usage (const char*);

static char* extensions[] = {
	"wav", "aiff",
};

static char* default_basename = "sample";
static char* default_ext = "wav";
static char *buffilename, *filename, *filepath;

void bp(){}

void Config_init (int* c, char*** v)
{
  table_init();
  set_defaults();
  parse_options (c, v);
  init_files();
}

inline const char* Config_sget (const char* p)
{
  return table_get (p);
}

inline int Config_iget (const char* p)
{
  return atoi (table_get (p));
}

inline void Config_iset (const char* p, int i)
{
  char buf[16];
  sprintf (buf, "%d", i);
  table_set (p, buf);
}

inline const char* Config_buffilename()
{
  assert (buffilename);
  return buffilename;
}

inline const char* Config_filename()
{
  assert (filename);
  return filename;
}

inline void Config_close()
{
  table_close();
  free (filename);
  free (buffilename);
}

//--------------------------------statics----------------------

static void set_defaults()
{
  table_set ("NO_GUI",        "0");
  table_set ("BASENAME",      "");
  table_set ("SOUND_BITS",    "8");
  table_set ("SOUND_SPEED",   "22050");
  table_set ("SOUND_CHANNELS","1");
  table_set ("FILE_FORMAT",   "");
  table_set ("TIMER",         "0");
  table_set ("INVISIBLE",     "0");
  table_set ("FORCE_ACTION",  "0");
}

static void parse_options (int* c, char*** v)
{
  int ch, b, len, argc, ind;
  char **argv;
  static struct option long_options[] = {
  	{ "force", no_argument, NULL, 'f' },
  	{ "help", no_argument, NULL, 'h' },
  	{ "invisible", no_argument, NULL, 'i' },
  	{ "no-gui", no_argument, NULL, 'n' },
  
  	{ "bits", required_argument, NULL, 'b' },
  	{ "channels", required_argument, NULL, 'c' },
  	{ "rate", required_argument, NULL, 'r' },
  	{ "timer", required_argument, NULL, 't' },
  	
  	{ 0, 0, 0, 0 }
  };

  const char* s = (*v)[0];
  len = strlen (s);
  b = strcmp (s+len-5, "yarec");
  table_set ("RECORDING", b? "0":"1");
  table_set ("FORCE_ACTION", b? "1":"0");

#ifdef X_WILL_BE_LINKED
  if (XOpenDisplay (0)) 
  	UIHandler_parse_gui_options (c, v);
  else
  	table_set ("NO_GUI", "1");
#endif
  argc = *c;
  argv = *v;
  
  while ((ch = getopt_long (argc, argv, 
                            "b:c:r:ft:nih?", long_options, (int*)0))
         != EOF)
         
  	switch (ch) {
	case '?': 
	case 'h': 
		usage(argv[0]);
		break;
	case 'f':
		table_set ("FORCE_ACTION", "1");
		break;
	case 'i':
		table_set ("INVISIBLE", "1");
		table_set ("FORCE_ACTION", "1");
		break;
	case 'n':
		table_set ("NO_GUI", "1");
		break;
	
	case 'b':
		b = atoi (optarg);
		if (b!=8 && b!=16 && b!=32) {
			fprintf (stderr, _("Invalid bits value: %s\n"), optarg);
			usage (argv[0]); 
		}
		table_set ("SOUND_BITS", optarg); 
		break;
	case 'c':
		b = atoi (optarg);
		if (b<1 && b>50) {
			fprintf (stderr, _("Invalid channels value: %s\n"), optarg);
			usage (argv[0]); 
		}
		table_set ("SOUND_CHANNELS", optarg); 
		break;
	case 'r': 
		b = atoi (optarg);
		if (b<1000 || b>51000) {
			fprintf (stderr, _("Invalid rate value: %s\n"), optarg);
			usage (argv[0]);
		}
		table_set ("SOUND_SPEED", optarg); 
		break;
	case 't':
		table_set ("TIMER", optarg); 
		table_set ("FORCE_ACTION", "1");
		break;
  	}

  parse_filename (argv[optind]);
  
}

static void parse_filename (const char* s)
{
  const char* b;
  char* p;
  int i;
  
  if (!s) {
  	if (!Config_iget ("RECORDING"))
  		Exception_fatal (_("No filename given for playing."));
  		
  	table_set ("BASENAME", default_basename);
  	table_set ("FILE_FORMAT", default_ext);
  	filename = malloc (strlen (default_basename) +
  	                   strlen (default_ext) + 2);
  	strcpy (filename, default_basename);
  	strcat (filename, ".");
  	strcat (filename, default_ext);
  	filepath = "";
  	return;
  }

  filename = malloc (strlen(s) + 1);
  strcpy (filename, s);
  
  b = basename (s);
  filepath = malloc (b - s + 1);
  strncpy (filepath, s, b - s);
  filepath [b-s] = '\0';
  
  p = (char*)b + strlen (b) - 1;
  while (p > b)
  	if (*p-- == '.') 
  		break;
  	
  if (++p == b) {
  	table_set ("BASENAME", b);
  	table_set ("FILE_FORMAT", default_ext);
  	return;
  }
  
  for (i=0; i<(sizeof(extensions)/sizeof(char*)); i++)
  	if (!strcmp (p+1, extensions[i])) {
  		table_set ("FILE_FORMAT", p+1);
  		*p = '\0';
  		table_set ("BASENAME", b);
  		return;
  	}
  
  Exception_fatal (strcatalloc (_("Unknown extension: "), p+1));
}

static void init_files()
{
  buffilename = (char*)strcatalloc (filepath, "yarec.tmp");
  remove (buffilename);

  if (!Config_iget ("RECORDING")) {
  	int r, b, c;
  	char buf [16];
  	struct File *f;

  	f = (struct File*) File_init (filename);
  	if (!File_ok (f))
  		Exception_fatal (strcatalloc (_("Couldn't read: "), filename));
  	Format_init (f, 0);
  	Format_get_params (&r, &b, &c);
  	Format_close();
  	SoundHandler_init (&r, &b, &c, 1); /* This to bail out if sth's wrong */
  	SoundHandler_close();
  	sprintf (buf, "%d", r);
  	table_set ("SOUND_SPEED", buf);
  	sprintf (buf, "%d", b);
  	table_set ("SOUND_BITS", buf);
  	sprintf (buf, "%d", c);
  	table_set ("SOUND_CHANNELS", buf);
  	File_close (f);
  }
}

static void usage (const char *progname)
{
// separate behaviour for record/play
// " [file basename] [-b sound_bits] [-r sound_rate] [-f file_format]"
    
  Exception_normal (_("\
  Usage: yarec  [options] [filename]\n\
               yaplay [options] filename\n\
==Options with arguments:\n\
  -r N, --rate=N      sampling rate in Hz (8000 etc.)\n\
  -b N, --bits=N      sample size in bits (8,16,32)\n\
  -c N, --channels=N  n of channels (1=mono, 2=stereo)\n\
  -t N, --timer=N     play/record for N seconds; sets -f\n\
==Options without arguments:\n\
  -f, --force         play/record immediately (default for yaplay)\n\
  -n, --no-gui        don't start GUI if in X\n\
  -i, --invisible     no visible output; sets -f; useful for scripts\n\
  -h, -?, --help      this text\n\
"));  
}

//--------------------------table-----------------------------------------

/* If you want to add keys, change config.gperf  */
/* You need gperf 2.7 for this to work. */

#include "table.h"

/* END OF GENERATED CODE */
/* ----------------------------- */

static char* table[TOTAL_KEYWORDS];

static void table_init()
{
}

static void table_set (const char* key, const char* value)
{
  const char* v;
  char* s;
  const struct entry* e;
  
  assert (key);
  assert (value);
  e = in_word_set (key, strlen(key));

  assert (e);
  v = table [e->value];
  if (v && !strcmp (v, value)) return;
  if (v) free ((char*)v);
  s = malloc (strlen (value) + 1);
  strcpy (s, value);
  table [e->value] = s;
}

static const char* table_get (const char* key)
{
  const struct entry* e;

  assert (key);
  e = in_word_set (key, strlen(key));

  assert (e); 
  assert (e->value);  /* This bombs if you try to get cfg before set */
  return table [e->value];
}

static void table_close()
{
  int i;
  for (i=0; i<TOTAL_KEYWORDS; i++)
  	if (table[i]) {
  		free (table[i]);
  		table[i] = 0;
  	}
}

